package com.mx.cneeds.server.device.web;

import java.util.*;

import com.mx.cneeds.common.dto.DeviceInfoDto;
import com.mx.cneeds.common.dto.DevicesDto;
import com.mx.cneeds.common.pager.PageUtils;
import com.mx.cneeds.common.result.R;
import com.mx.cneeds.common.validator.ValidatorUtils;
import com.mx.cneeds.server.device.service.DeviceInfoService;
import com.mx.cneeds.server.entity.DeviceInfoEntity;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;



/**
 * 设备信息
 *
 * @author Mark
 * @email sunlightcs@gmail.com
 * @date 2020-03-18 14:44:30
 */
@RestController
@RequestMapping("device/info")
public class DeviceInfoController {
    @Autowired
    private DeviceInfoService deviceInfoService;

    /**
     * 列表
     */
    @RequestMapping("/list")
    public PageUtils list(@RequestParam Map<String, Object> params){
        PageUtils page = deviceInfoService.queryPage(params);

        return page;
    }


    /**
     * 信息
     */
    @RequestMapping("/info/{deviceId}")
    public R info(@PathVariable("deviceId") Long deviceId){
        DeviceInfoEntity deviceInfo = deviceInfoService.getById(deviceId);

        return R.ok().put("deviceInfo", deviceInfo);
    }

    /**
     * 信息
     */
    @RequestMapping("/infoByImei")
    public DeviceInfoDto infoByimei(String imei){
        DeviceInfoEntity deviceInfoEntity = deviceInfoService.queryDeviceByImei(imei);
        DeviceInfoDto deviceInfoDto = new DeviceInfoDto();
        if (deviceInfoEntity != null){
            BeanUtils.copyProperties(deviceInfoEntity,deviceInfoDto);
            return deviceInfoDto;
        }else{
            return null;
        }

    }

    @RequestMapping("/infoByIds")
    public List<DeviceInfoDto> infoByIds(@RequestBody Set<Long> ids){
        List<DeviceInfoEntity> deviceInfoEntitys = deviceInfoService.queryDeviceByIds(ids);

        List<DeviceInfoDto> deviceInfoDtos = new ArrayList<>();
        deviceInfoEntitys.forEach(deviceInfoEntity -> {
            DeviceInfoDto deviceInfoDto = new DeviceInfoDto();
            BeanUtils.copyProperties(deviceInfoEntity,deviceInfoDto);
            deviceInfoDtos.add(deviceInfoDto);
        });
        return deviceInfoDtos;
    }

    /**
     * 保存
     */
    @RequestMapping("/save")
    public R save(@RequestBody DeviceInfoEntity deviceInfo){
        deviceInfoService.save(deviceInfo);

        return R.ok();
    }

    /**
     * 修改
     */
    @RequestMapping("/update")
    public R update(@RequestBody DeviceInfoEntity deviceInfo){
        ValidatorUtils.validateEntity(deviceInfo);
        deviceInfoService.updateById(deviceInfo);
        
        return R.ok();
    }

    /**
     * 删除
     */
    @RequestMapping("/delete")
    public R delete(@RequestBody Long[] deviceIds){
        deviceInfoService.removeByIds(Arrays.asList(deviceIds));

        return R.ok();
    }

    /**
     * 保存
     */
    @RequestMapping("/importDevice")
    public R importDevice(@RequestBody DevicesDto devicesDto){
        List<DeviceInfoEntity> deviceInfoEntityList = new LinkedList<>();
        if (devicesDto.getImeis() != null && devicesDto.getImeis().size() != 0) {
            devicesDto.getImeis().forEach(str -> {
                DeviceInfoEntity deviceInfoEntity = new DeviceInfoEntity();
                deviceInfoEntity.setImei(str);
                deviceInfoEntity.setSeriesId(devicesDto.getSeriesId());
                deviceInfoEntity.setDeptId(devicesDto.getDeptId());
                deviceInfoEntity.setCreateTime(new Date());
                deviceInfoEntityList.add(deviceInfoEntity);
            });
        }
        deviceInfoService.saveBatch(deviceInfoEntityList);
        return R.ok();
    }


    /**
     * 保存
     */
    @RequestMapping("/updateDeviceSeriesBatch")
    public R updateDeviceSeriesBatch(@RequestBody DevicesDto devicesDto){
        List<DeviceInfoEntity> deviceInfoEntityList = new LinkedList<>();
        if (devicesDto.getImeis() != null && devicesDto.getImeis().size() != 0) {
            devicesDto.getImeis().forEach(str -> {
                DeviceInfoEntity deviceInfoEntity = new DeviceInfoEntity();
                deviceInfoEntity.setImei(str);
                deviceInfoEntity.setSeriesId(devicesDto.getSeriesId());
                deviceInfoEntity.setDeptId(devicesDto.getDeptId());
                deviceInfoEntityList.add(deviceInfoEntity);
            });
        }
        deviceInfoService.updateBatchByImei(deviceInfoEntityList);

        return R.ok();
    }

    /**
     * 保存
     */
    @RequestMapping("/updateDeviceChannelBatch")
    public R updateDeviceChannelBatch(@RequestBody DevicesDto devicesDto){
        List<DeviceInfoEntity> deviceInfoEntityList = new LinkedList<>();
        if (devicesDto.getImeis() != null && devicesDto.getImeis().size() != 0) {
            devicesDto.getImeis().forEach(str -> {
                DeviceInfoEntity deviceInfoEntity = new DeviceInfoEntity();
                deviceInfoEntity.setImei(str);
                deviceInfoEntity.setChannelNums(devicesDto.getChannelNums());
                deviceInfoEntity.setChannelNumsMessage(devicesDto.getChannelNumsMessage());
                deviceInfoEntity.setChannelRules(devicesDto.getChannelRules());
                deviceInfoEntityList.add(deviceInfoEntity);
            });
        }
        deviceInfoService.updateBatchByImei(deviceInfoEntityList);

        return R.ok();
    }
}
